<?php

declare(strict_types=1);

namespace Reloadify\Service;

use Reloadify\Event\AbandonedCartDeleteEvent;
use Shopware\Core\Checkout\Cart\AbstractCartPersister;
use Shopware\Core\Checkout\Cart\Cart;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Symfony\Contracts\EventDispatcher\EventDispatcherInterface;

class RedisCartPersisterDecorator extends AbstractCartPersister
{
    private AbstractCartPersister $cartPersister;
    private EventDispatcherInterface $eventDispatcher;

    public function __construct(AbstractCartPersister $cartPersister, EventDispatcherInterface $eventDispatcher)
    {
        $this->cartPersister = $cartPersister;
        $this->eventDispatcher= $eventDispatcher;
    }

    public function getDecorated(): AbstractCartPersister
    {
        return $this->cartPersister;
    }

    public function replace(string $oldToken, string $newToken, SalesChannelContext $context): void
    {
        $this->getDecorated()->replace($oldToken, $newToken, $context);
    }

    public function load(string $token, SalesChannelContext $context): Cart
    {
        return $this->getDecorated()->load($token, $context);
    }

    public function save(Cart $cart, SalesChannelContext $context): void
    {
        $this->getDecorated()->save($cart, $context);
    }

    public function delete(string $token, SalesChannelContext $context): void
    {
        $this->eventDispatcher->dispatch(new AbandonedCartDeleteEvent($token,$context));
        $this->getDecorated()->delete($token, $context);
    }
}
